// File: test.cc
//
// Created: Fri Dec 26 13:04:18 2003
//
// $Id: sdb_shell.cc,v 1.4 2004/04/21 11:54:06 sav Exp $
//

#include<string>
#include<iostream>

#include "../SDB.h"

void print(ll_tb_iter f, ll_tb_iter s, table<noid_type, noid_type> tbl, const char* nrif_top = NULL) {
  std::cout << "Found: \n";
  int i=0;
  for (ll_tb_iter ref = f; ref != s; ++ref, i++) {
    std::cout << "       v[" << i <<"]=" << tbl.get_val(ref.get_cur()) << " --> ("  
	      << ref.get_key() << ") " ;
    //if (nrif_top != NULL) std::cout << nrif_top + tbl.get_key(ref.get_cur()) ;
    std::cout << "\n";
  }
}

void test_find(NodeRIndex<noid_type, noid_type> obj) {
  if(obj.get_valtype() == NodeConst::STRING4) {
    std::string s;
    std::cin >> s;
    std::pair<ll_tb_iter, ll_tb_iter> p = obj.get_find_map().map_string_key(s.c_str(),NodeConst::PREFIX);
    if (p.first.is_end()) std::cout << "Not found: " << s << "\n";
    else {
      print(p.first, p.second, obj.get_find_map(), (const char*) obj.get_top());
    }
  } else if (obj.get_valtype() == NodeConst::INT4) {
    noid_type noid;
    std::cin >> noid;
    std::pair<ll_tb_iter, ll_tb_iter> p = obj.get_find_map().map_numeric_key(noid);
    if (p.first.is_end()) std::cout << "Not found: " << noid << "\n";
    else {
      print(p.first, p.second, obj.get_find_map());
    }
  }
}

void test_get(NodeRIndex<noid_type, noid_type> obj) {
    const char* nrif_top = NULL;
    if(obj.get_valtype() == NodeConst::STRING4)  nrif_top = (const char*)obj.get_top();

    noid_type noid;
    std::cin >> noid;
    std::pair<ll_tb_iter, ll_tb_iter> p = obj.get_get_map().map_numeric_key(noid);
    if (p.first.is_end()) std::cout << "Not found: " << noid << "\n";
    else { print(p.first, p.second, obj.get_get_map(), nrif_top ); }
}

void print_nrif(NodeRIndex<noid_type, noid_type> obj) {
  std::cout << "Key Type: " << obj.get_keytype() << " (" << NodeConst::get_type_name(obj.get_keytype()) << ")\n";
  std::cout << "Val Type: " << obj.get_valtype() << " (" << NodeConst::get_type_name(obj.get_valtype()) << ")\n";
  std::cout << "Size: " << obj.get_size() << "\n";

  std::cout << "FLAGS:\n";
  std::cout << " isGetUnique: " << obj.isGetUnique() << "\n";
  std::cout << " isFindUnique: " << obj.isFindUnique() << "\n";
  std::cout << " hasDefVal: " << obj.hasDefVal() << "\n";

  table<noid_type, noid_type> f = obj.get_find_map(); 
  table<noid_type, noid_type> s = obj.get_get_map(); 
  
  const char *top = (const char *) obj.get_top();
  std::cout << "FIND MAP:";
  if (f.get_data_offset()==NULL) {
    std::cout << " none\n";
  } else {
    std::cout << "\n";
    for(ll_tb_iter ref = f.begin(); ref != f.end(); ++ref) {
      std::cout << "(" << ref.get_key();
      if(obj.get_valtype() == NodeConst::STRING4) std::cout << " = " << top + ref.get_key() ;
      std::cout << ") -> " << obj.get_find_map().get_val(ref.get_cur());
      std::cout << "\n";
    }
  }

  std::cout << "GET MAP:\n";
  for(ll_tb_iter ref = s.begin(); ref != s.end(); ++ref) {
    std::cout << ref.get_key();
    std::cout << "  -> " << "(" << obj.get_get_map().get_val(ref.get_cur());
    if(obj.get_valtype() == NodeConst::STRING4) std::cout << " = " << top + obj.get_get_map().get_val(ref.get_cur()) ;
    std::cout << ")" << "\n";
  }
}

int main(int argc, char **argv) {
  std::string filename;
  if (argc != 2) {
    std::cout << "Usage: sdb_shell <sdb_file>\n";
    exit(0);
  } 
  try {
    sdb_header nr(argv[1]);
    std::cout << nr.info();
    NodeRIndex<noid_type, noid_type> *tab = nr.get_nrif();

    std::cout << "Number of elements: " << tab->get_size() << "\n";

    print_nrif(*tab);

    noid_type noid;
    std::cin >> noid;
    if (! tab->get_valtype()==NodeConst::CONTEXT4 ) { throw sdb_error("Bad noid found!"); } 
    else {
      std::cerr << "Type: " << tab->get_valtype() << " (" << NodeConst::get_type_name(tab->get_valtype()) << ")\n";
      std::pair< ll_tb_iter, ll_tb_iter > res = tab->get_get_map().map_numeric_key(noid);
      if (res.first.is_end()) throw sdb_error("Not found !");
      if (res.second - res.first > 1) throw sdb_error("Multi context ?");
      NodeRIndex<noid_type, noid_type> obj(tab->get_top()+ tab->get_get_map().get_val(res.first.get_cur()));
      print_nrif(obj);
      std::cout  << "Command [<key> -- cd to node, g<key> -- get, f<key>] -- find: ";
      char com;
      while(std::cin >> com) {
	if (com == 'g') {
	  test_get(obj);
	} else if (com == 'f') {
	  test_find(obj);
	}
	std::cout  << "Command: ";
      }
    }
  } catch (sdb_error e) {  
    std::cerr << "Error: " << e.what() << "\n";
  }
}


//  Local Variables:
//  mode: c++
//  End:
