#ifndef _TB_ITER_H_
#define _TB_ITER_H_

#include "sdb_error.h"

template<class TD1> 
class tb_iter {
private:
  const TD1 *tbl;
  size_t border;
  size_t cur;
public:
  tb_iter(const TD1* ctbl, size_t cborder, size_t ccur=0) : tbl(ctbl), border(cborder), cur(ccur) {
    if (ccur<0 || ccur>border) throw sdb_error("tb_iter: bad index");
  }

  friend bool operator==(const tb_iter& a, const tb_iter& b) {
    return a.cur==b.cur;
  }

  friend bool operator!=(const tb_iter& a, const tb_iter& b) {
    return a.cur!=b.cur;
  }

  friend tb_iter half(const tb_iter& a, const tb_iter& b) {
    return tb_iter(a.tbl, a.get_size(), (a.cur+b.cur)/2);
  }

  friend size_t operator-(const tb_iter& a, const tb_iter& b) {
    return a.cur-b.cur;
  }

  bool is_end(void) const { return cur==border; }

  size_t get_cur(void) const { return cur; };
  size_t get_size(void) const { return border; };

  tb_iter& operator++(void);
  tb_iter& operator--(void);

  inline TD1 get_key(void) const { return tbl[cur]; };
  TD1 get_key_checked(void) const;
};

template<class TD1>
inline tb_iter<TD1>& tb_iter<TD1>::operator++(void) {
  if (cur < border) cur++;
  return *this;
}

template<class TD1>
inline  tb_iter<TD1>& tb_iter<TD1>::operator--(void) {
  if (cur>0) cur--;
  return *this;
}

template<class TD1>
inline TD1 tb_iter<TD1>::get_key_checked(void)  const {
  if (cur < border) return get_key();
  printf("key: Cur = %ld from %zd \n" , cur, border);
  throw sdb_error("(tb_iter):checked:Iter: bad index");
}

#endif

//  Local Variables:
//  mode: c++
//  End:
