// File: cmps_light.h
//
// Created: Mon Feb  2 17:20:57 2004
//
// $Id: cmps_light.h,v 1.1 2004/02/19 16:57:39 igor Exp $
//

#ifndef __TYPEDEFS_H__
#define __TYPEDEFS_H__

typedef long long large_int;
typedef int noid_type;
typedef unsigned int uint4;
typedef int int4;
typedef unsigned char flag;
typedef char byte;
typedef int offset_type;


#define TWO_POWER_32LL 4294967296LL
#if __GNUC__ > 5
 #define GNU_CONST_STATIC_FLOAT_DECLARATION constexpr
#else
 #define GNU_CONST_STATIC_FLOAT_DECLARATION const
#endif

class int8 {
  // not stric c++: 'constexpr' needed for in-class initialization of static data member ...
  // const static double TWO_POWER_32_DOUBLE = 4294967296.0;
  GNU_CONST_STATIC_FLOAT_DECLARATION static double TWO_POWER_32_DOUBLE = 4294967296.0;
public:
  uint4 low;
  uint4 high;
  int8(long long l) :   low(l%TWO_POWER_32LL), high(l/TWO_POWER_32LL) {}
  int8(uint4 h=0, uint4 l=0) :   low(l), high(h) {}
  int8(double d) {
    high = (uint4)(d/TWO_POWER_32_DOUBLE);
    low = (uint4)(d-high*TWO_POWER_32_DOUBLE);
  }
  long long to_long_long(void) const {return high*TWO_POWER_32LL +low;} //4294967296=2^32

  friend bool operator<(const int8 &a, const int8 &b) {
    if(a.high<b.high) return true;
    if(a.high>b.high) return false;
    if(a.low<b.low) return true;
    return false;
  }
  friend bool operator>(const int8 &a, const int8 &b) {
    return b<a;
  }
  operator double(void) const {
    return high*TWO_POWER_32_DOUBLE+low;
  }
};

#define NUM_OF_TYPES 9

class NodeConst {
public:
  enum comparison_type {PREFIX, FULL};
  enum TYPE {INT1, INT2, INT4, INT8, STRING4, STRING8, CONTEXT4, CONTEXT8, DOUBLE};
  enum NRIF_FLAGS {GETUNIQUE=1, FINDUNIQUE=2, HASDEFVAL=4};

  static const char *type_name[NUM_OF_TYPES];
  static const char *flags_name[3];

  static bool is_valid_type(TYPE t) {
    if (t>=0 && t< NUM_OF_TYPES) return true;
    return false;
  }

  static const char* get_type_name(TYPE t) {
    if (is_valid_type(t)) return type_name[t];
    return "Unknown type";
  }
};

const char* NodeConst::type_name[NUM_OF_TYPES]= {"INT1","INT2","INT4", "INT8", 
				      "STRING4", "STRING8", "CONTEXT4", "CONTEXT8", "DOUBLE"};
const char* NodeConst::flags_name[3]= {"GET_UNIQUE", "FIND_UNIQUE", "HAS_DEFVAL"};


#endif

//  Local Variables:
//  mode: c++
//  End:
